; Main source file for 3D Checkers
; Copyright 2000-2001 Joshua Seagoe

#include "asm86.h"
#include "ti86asm.inc"
#include "gray3equ.inc"

dark_plane equ $8400
light_plane equ $8800

KING_BIT equ %00000010
PLAYER_BIT equ %00000001

#include "sprites.inc"


.org $d748

    nop
;    jp menu_done
    jp start

    .dw 0
    .dw title

title:
    .db "3D Checkers by Joshua Seagoe",0

start:
    call _runindicoff
;    call _clrLCD

#if 0
; get writeback area
;    rst 18h
    rst 10h         ; get program var
    push bc
    push de
    ld hl,savegame
    xor a
    add hl,de
    adc a,b
    ; ahl = ptr to savegame data
    
    pop hl
    pop af
    ld de,sprites
    add hl,de
    adc a,0
    ; ahl = ptr to sprite data
    
    ld a,$40
    out ($06),a
#endif

; no game type select in this version

#if 0
    ld hl,gametype
    ld de,$fc00
    call Disp86i

    ld bc,$0402
menu_loop:
    ld (_curRow),bc
    ld a,LblockArrow
    call _putmap
    call _getkey
    push af
    ld a,' '
    call _putmap
    pop af
    cp kEnter
    jr z,menu_done
    cp kExit
    jr z,menu_quit
    cp kUp
    jr z,menu_up
    cp kDown
;    jr z,menu_down
    jr nz,menu_loop
menu_down:
    inc c
    ld a,c
    cp 2+4
    jr nz,menu_loop
menu_up:
    dec c
    ld a,c
    cp 2-1
    jr nz,menu_loop
    jr menu_down
menu_quit:
    call _clrScrn
    ret

menu_done:
   ; c = 2 1 player
   ;     3 2 player, 1 calc
   ;     4 2 player, 2 calc
   ;     5 exit
   ld a,c
   sub 2
   jr z,p1_start
   dec a
   jr z,p2c1_start
   dec a
   jr z,p2c2_start
   jr menu_quit


p1_start:
    ld hl,comp_player
    ld (player_2_patch),hl
#endif
p2c2_start:
p2c1_start:
   ; unpack sprites
    ld hl,sprites
    ld de,sprite_base
    call Disp86i

   ; set up grayscale
    call clear_screen
    call gray_install

new_game:

    ld hl,init_board
    ld de,board
    ld bc,9*8
    ldir

    ld a,$3c
    ld (cursor),a
    xor a
    ld (select),a

    ld a,1                      ; 1=red/0=black
    ld (player),a
    ld (perspective),a

main_loop:
    xor a                       ; clear selection
    ld (select),a
main_get_move:
    push bc
    ld hl,user_player
player_1_patch = $-2
    ld de,(player_2_patch)
    ld (player_2_patch),de
    ld (player_1_patch),hl
    call exec_player
    pop bc
    jr z,main_exit
    dec a
    jr z,not_jump
    push af
    ld a,(select)               ; remove jumped piece
    ld b,a
    cp c
    jr z,double_jump_ok
    ld a,c
    or a
    jr z,double_jump_ok
    pop af
    jr main_get_move
double_jump_ok:
    ld a,(cursor)
    add a,b
    rra
    ld e,a
    xor a
    ld d,a
    push hl
    ld hl,board
    add hl,de
    ld (hl),a
    pop hl
    jr is_jump
not_jump:
    cp c                        ; is c 0?
    jr nz,main_get_move         ; no, we're on a double jump
    push af
is_jump:
    ld a,(select)               ; move selected piece
    ld e,a
    ld d,0
    push hl
    ld hl,board
    add hl,de
    pop de
    ldi
    dec hl
    ld (hl),0
    ld a,(cursor)               ; top row?
    ld (select),a
    cp 8
    jr nc,dont_king
    dec de
    ex de,hl
    ld a,(hl)
    or KING_BIT                 ; make piece a king
    ld (hl),a
dont_king:
    pop af
    jr z,not_jump1
    ld a,(cursor)               ; check for double jump
    ld c,a                      ; mark double jump
    call check_jump
    jr nz,main_get_move
not_jump1:

    ld a,(player)
    xor 1
    ld (player),a
    ld bc,0
    jr main_loop

main_exit:
    call clear_screen
    call copy_screen
    call gray_remove
    call _clrLCD
    ret

flush_buffer:
    ; read all events
    halt
    call gray_get_event
    jr nz,flush_buffer
    ; wait for all keys up
    ld b,7
    ld hl,gray_keys
    sub a
flush_buffer1:
    or (hl)
    inc hl
    djnz flush_buffer1
    or a
    jr nz,flush_buffer
    ret

draw_board:
    call clear_screen

    ld hl,boardd                ; dark plane
    ld de,dark_plane
    call Disp86i
    ld hl,boardl                ; light plane
    ld de,light_plane
    call Disp86i

; draw select
    ld a,(select)
    or a
    jr z,no_draw_select
    ld l,a
    ld h,0
    add hl,hl
    ld de,board_table
    add hl,de
    ld e,(hl)
    inc hl
    ld d,(hl)
    ld hl,$0030-1
    add hl,de
    push hl
    ld de,select_left
    call xor_sprite
    pop hl
    inc hl
    inc hl
    ld de,select_right
    call xor_sprite
    
no_draw_select:
    call draw_pieces
    jp copy_screen

mask_sprite:
    ld a,(de)
    ld b,a
    inc de
    push hl
    push bc
    push hl
    push de
    call mask_sprite1
    pop de
    pop hl
    ld bc,$0400
    add hl,bc
    pop bc
    push bc
    call mask_sprite1
    pop bc
    pop hl
    jr xor_sprite2

; de=sprite
; hl=screen
mask_sprite1:
    ld a,(de)
    and (hl)
    ld (hl),a
    inc de
    inc hl
    ld a,(de)
    and (hl)
    ld (hl),a
    inc de
    push de
    ld de,$0010-1
    add hl,de
    pop de
    djnz mask_sprite1
    ret
    
xor_sprite:
    ld a,(de)
    ld b,a
    inc de
xor_sprite2:
    push bc
    push hl
    call xor_sprite1            ; dark plane
    pop hl
    ld bc,$0400
    add hl,bc                   ; light plane
    pop bc
xor_sprite1:
    ld a,(de)
    xor (hl)
    ld (hl),a
    inc de
    inc hl
    ld a,(de)
    xor (hl)
    ld (hl),a
    inc de
    push de
    ld de,$0010-1
    add hl,de
    pop de
    djnz xor_sprite1
    ret

;these are now compressed in sprites.asm

;piece_table:
;    .dw black_man,red_man,black_king,red_king
;    .dw pointer,pointer_half,pointer,pointer_half
;
;
;board_table:
;    ;       1     2     3     4     5     6     7     8
;    .dw $ebd7,$ec16,$ec55,$ec94,$ecd3,$ed12,$ed51,$ed90 ; 1
;    .dw $ec18,$ec57,$ec96,$ecd5,$ed14,$ed53,$ed92,$edd1 ; 2
;    .dw $ec59,$ec98,$ecd7,$ed16,$ed55,$ed94,$edd3,$ee12 ; 3
;    .dw $ec9a,$ecd9,$ed18,$ed57,$ed96,$edd5,$ee14,$ee53 ; 4
;    .dw $ecdb,$ed1a,$ed59,$ed98,$edd7,$ee16,$ee55,$ee94 ; 5
;    .dw $ed1c,$ed5b,$ed9a,$edd9,$ee18,$ee57,$ee96,$eed5 ; 6
;    .dw $ed5d,$ed9c,$eddb,$ee1a,$ee59,$ee98,$eed7,$ef16 ; 7
;    .dw $ed9e,$eddd,$ee1c,$ee5b,$ee9a,$eed9,$ef18,$ef57 ; 8
;    .dw $eddf,$ee1e,$ee5d,$ee9c,$eedb,$ef1a,$ef59,$ef98

; .B.B.B.B
; B.B.B.B.
; .B.B.B.B
; -.-.-.-.
; .-.-.-.-
; R.R.R.R.
; .R.R.R.R
; R.R.R.R.

;init_board:
;    .db  0 ,$10, 0 ,$20, 0 ,$30, 0 ,$40
;    .db $50, 0 ,$60, 0 ,$70, 0 ,$80, 0
;    .db  0 ,$90, 0 ,$a0, 0 ,$b0, 0 ,$c0
;    .db $00, 0 ,$00, 0 ,$00, 0 ,$00, 0
;    .db  0 ,$00, 0 ,$00, 0 ,$00, 0 ,$00
;    .db $c1, 0 ,$b1, 0 ,$a1, 0 ,$91, 0
;    .db  0 ,$81, 0 ,$71, 0 ,$61, 0 ,$51
;    .db $41, 0 ,$31, 0 ,$21, 0 ,$11, 0
;    .db  0 , 0 , 0 , 0 ,$04, 0 , 0 , 0

rotate_board:
    ld de,board+1
    ld hl,board+62
    ld b,2
rotate_board1:
    push bc
    ld b,4
rotate_board2:
    ld c,(hl)
    ld a,(de)
    ex de,hl
    ld (hl),c
    ld (de),a
    ex de,hl
    inc de
    inc de
    dec hl
    dec hl
    djnz rotate_board2
    dec de
    inc hl
    ld b,4
rotate_board3:
    ld c,(hl)
    ld a,(de)
    ex de,hl
    ld (hl),c
    ld (de),a
    ex de,hl
    inc de
    inc de
    dec hl
    dec hl
    djnz rotate_board3
    inc de
    dec hl
    pop bc
    djnz rotate_board1
    ld a,(perspective)
    xor 1
    ld (perspective),a
    ret

draw_pieces:
    ld de,board+1
    ld hl,board_table+2

    ld b,9*8-1
draw_pieces1:
    push bc
    ld a,(de)                   ; a = piece code
    inc de
    push de
    ld e,(hl)                   ; de = screen
    inc hl
    ld d,(hl)
    inc hl
    push hl

    or a
    jr z,draw_pieces2           ; empty

    push de
    and $07
    ld e,a
    ld d,0
    ld hl,piece_table
    add hl,de
    add hl,de
    ld e,(hl)                   ; de = piece sprite
    inc hl
    ld d,(hl)
    pop hl                      ; hl = screen
    call mask_sprite

draw_pieces2:
    pop hl
    pop de
    pop bc
    djnz draw_pieces1
    ret

;p1name:; 12345678
;    .db "Black",0
;p2name:
;    .db "White",0
;
;loses:
;    .db " loses",0

;bwin:   .db "Black Wins",0
;wwin:   .db "White Wins",0
;pwin:   .db "You Win",0
;cwin:   .db "You Lose",0

;cexit:  .db "Really Quit?",0

write_turn:
    push hl
    ld hl,52*256+103 ; (63-7-4),(127-20-4)
    ld (_penCol),hl
    ld a,(player)
    or a
    jr z,p1turn
    ld hl,p2name
    jr p2turn
p1turn:
    ld hl,p1name
p2turn:
    call _vputs
    pop hl
    ret

clear_screen:
    ld hl,light_plane
    ld de,light_plane+1
    ld bc,$0400-1
    ld (hl),l
    ldir
    ld hl,dark_plane
    ld de,dark_plane+1
    ld bc,$0400-1
    ld (hl),l
    ldir
    ret

copy_screen:
    ld hl,dark_plane
    ld de,gray_dark_plane
    ld bc,$0400
    ldir
    ld de,gray_light_plane
    ld bc,$0400
    ldir
    call write_turn
    ret

; **************************************************************************

;call: e=index
;ret:  a=piece
;      d=0
get_piece:
    ld a,e
    cp 64
    jr nc,get_piece0
    ld d,0
    push hl
    ld hl,board
    add hl,de
    ld a,(hl)
    pop hl
    ret
get_piece0:                     ; piece isn't on the board
    xor a
    ret

; can player move?
; a=0 no, player loses
; a=1 yes
; a=2 yes, player must jump
check_player:

    ; if you can jump, you must
    ld b,63
check_player_jump_loop:
    ld e,b
    call check_jump_p
    or a
    jr nz,check_player_jump     ; there is a jump available
    djnz check_player_jump_loop

    ; if you can move, you must
    ld b,63
check_player_move_loop:
    ld e,b
    call check_piece
    or a
    ret nz
    djnz check_player_move_loop

    ; if you can't move, you've lost
    ret

check_player_jump:
    ld a,2
    ret
check_piece_player:             ; wrong player
    xor a
    ret

; can piece at e move?
; a=0 no
; a=1 single

;preserves e,hl,bc
check_piece:
    call get_piece
    and %11111011
    ret z                       ; no piece, no move
    ld d,a

    ld a,(player)
    xor d                       ; set player bit if mismatch
    and PLAYER_BIT              ; isolate it
    jr nz,check_piece_player

    ld a,-7
    call check_piece_move
    ld a,-9
    call check_piece_move
    ld a,d
    and KING_BIT                ; kings move backwards
    ret z                       ; men don't
    ld a,7
    call check_piece_move
    ld a,9
    call check_piece_move
    xor a
    ret

check_piece_move:
    push de
    add a,e                     ; offset + piece loc
    ld e,a
    call get_piece
    pop de
    and %11111011               ; is the space empty?
    ret nz
    pop af                      ; don't return to check_piece
    ld a,1
    ret


; can piece at (select) move to (cursor)?
; a=0 fail
; a=1 single move
; a=2 jump
check_move:
    ld de,(cursor)              ; d=(select) e=(cursor)
    push de
    ld e,d
    call get_piece
    pop de
    and KING_BIT                ; king?
    ld a,e
    jr nz,check_move_king
    cp d                        ; no, cursor must be < select
    jr nc,check_move_fail
check_move_king:
    sub d                       ; difference between here and there
    cp -7
    jr z,check_move_single
    cp 7
    jr z,check_move_single
    cp -9
    jr z,check_move_single
    cp 9
    jr z,check_move_single
    cp -14
    jr z,check_move_jump
    cp 14
    jr z,check_move_jump
    cp -18
    jr z,check_move_jump
    cp 18
    jr z,check_move_jump
check_move_fail:
    xor a
    ret
check_move_single:              ; single move, check if clear
    push de
    call get_piece
    pop de
    and %11111011
    jr nz,check_move_fail
    inc a                       ; single move ok
    ret
check_move_jump:
    ld a,e
    add a,d
    rra                         ; a=middle point
    push de
    ld e,a
    call get_piece              ; check piece to jump over
    pop de
    and %11111011
    jr z,check_move_fail        ; no piece to jump over
    and PLAYER_BIT
    ld b,a
    ld a,(player)
    cp b                        ; enemy piece?
    jr z,check_move_fail
    push de                     ; check if clear
    call get_piece
    pop de
    and %11111011
    jr nz,check_move_fail
    add a,2                     ; jump ok
    ret

; also make sure there's a piece there
check_jump_p:
    call get_piece
    and %11111011               ; must be a piece there
    ret z
    ld d,a                      ; check for a piece
    ld a,(player)               ; belonging to the current player
    xor d
    and PLAYER_BIT
    jr z,check_jump_p2
    xor a
    ret
check_jump_p2:
    ld a,e

; does piece at a have a jump available?
; z=no, nz=yes
; destroys hl,de,a
check_jump:
    ld l,a
    ld a,(player)
    ld h,a
    ld a,-9
    call check_jump_dir
    ret nz
    ld a,-7
    call check_jump_dir
    ret nz
    ld e,l                      ; check original piece 
    push hl
    call get_piece
    and KING_BIT                ; only kings can jump backwards
    pop hl
    ret z
    ld a,9
    call check_jump_dir
    ret nz
    ld a,7
    call check_jump_dir
    ret

; z=invalid, nz=valid
; e=a
valid_space:
    ld e,a                      ; ONE of x,y must be odd
    cp 63                       ; outside of grid
    jr nc,not_valid
    rra
    rra
    rra                         ; (a)^(a/8)
    xor e
    bit 0,a                     ; valid if bit 0 is set
    ld a,e
    ret
not_valid
    cp a                        ; set zf
    ret

;in:
; l=piece, h=player
; a=direction
;out:
; z=can't jump, nz=can jump
check_jump_dir:
    push af
    add a,l
    ld e,a                      ; e = new piece loc
    call get_piece
    and %11111011
    jr z,check_jump_dir2        ; nothing to jump over
    and PLAYER_BIT              ; check player
    cp h
    jr nz,check_jump_dir1       ; fail if it's friendly piece
check_jump_dir2:
    pop af
check_jump_dir0:                ; fail
    xor a                       ; z=can't jump
    ret
check_jump_dir1:
    pop af
    add a,e                       ; check landing spot
    call valid_space
    jr z,check_jump_dir0          ; z=can't jump
;    ld e,a
    call get_piece
    and %11111011
    jr nz,check_jump_dir0
    inc a                       ; succeed
    ret                         ; nz=can jump

; **************************************************************************
; **************************************************************************
; **************************************************************************
; **************************************************************************

wait_for_event:
    halt
    call gray_get_event
    jr z,wait_for_event

    ; exit
    cp key_exit + gray_key_down
    jr z,confirm_exit

    ; menu 1
;    cp key_f1 + gray_key_down

    ; menu 2
 ;   cp key_f2 + gray_key_down
 
    cp key_plus + gray_key_down
    jr z, contrast_plus
    
    cp key_minus + gray_key_down
    jr z, contrast_minus

    ; switch gray shades
    cp key_times + gray_key_down
;    jr z,switch_grays

    ret nz

switch_grays:
    push af
    ld a,(gray_total_patch)
    xor %00000110   ; toggle 3 & 5
    ld (gray_total_patch),a
    ld a,(gray_split_patch)
    xor %00000011   ; toggle 1 & 2
    ld (gray_split_patch),a
    pop af
    ret
    
contrast_plus:


confirm_exit:
    pop hl              ; don't return to caller
    xor a               ; a=0
    ret

; **************************************************************************

user_keyup:
    cp key_2nd
    jr z,user_select
    cp key_enter
    jr z,user_select
    jr user_loop

user_player:
user_redraw:
    call draw_board
user_loop:
    call wait_for_event
    bit 6,a
    jr z,user_keyup
;    cp key_exit + gray_key_down
;    jr z,confirm_exit

    cp key_up + gray_key_down
    ld b,-9
    jr z,cursor_move

    cp key_down + gray_key_down
    ld b,9
    jr z,cursor_move

    cp key_left + gray_key_down
    ld b,-7
    jr z,cursor_move

    cp key_right + gray_key_down
    ld b,7
    jr z,cursor_move

    cp key_2nd + gray_key_down
    jr z,user_select
    cp key_enter + gray_key_down
    jr nz,user_loop

user_select:
    ld a,(cursor)
    ld e,a
    ld d,0
    ld hl,board
    add hl,de
    ld a,(hl)
    or a
    jr z,user_set_moveto
    and %00001101       ; get player code
    ld d,a
    ld a,(player)
    cp d
    jr nz,user_loop
user_set_select:
    ld a,e
    ld (select),a
user_redraw1:
    jr user_redraw
user_set_moveto:
    ld a,(select)
    or a
    jr z,user_loop
    call check_move
    jr z,user_loop
    ld bc,(jumpreq)             ; ld c,(jumpreq)
    cp c
    jr nz,user_loop
    or a
    ret

cursor_move
    ld a,(cursor)
    ld e,a
    ld d,0
    ld hl,board+8
    add hl,de
    ld (hl),d
    add a,b
    and $3f
    ld b,a
    srl a
    srl a
    srl a
    xor b
    and 1
    xor 1
    xor b
    ld (cursor),a
    ld b,4
    cp 7*8
    jr nz,cursor_notedge
    inc b
cursor_notedge
    ld e,a
    ld d,0
    ld hl,board+8
    add hl,de
    ld (hl),b
    jr user_redraw1


player_lost:
    call draw_board
    ld hl,0
    ld (_penCol),hl
    ld hl,p2name
    ld a,(player)
    or a
    jr nz,player_lost1
    ld hl,p1name
player_lost1:
    call _vputs
    
    ld hl,loses
    call message
    pop hl              ; ret to main loop
    pop hl
;    call gray_remove
    cp key_exit + gray_key_down
    jp z,main_exit
    jp new_game

message:
    call _vputs

    call flush_buffer
message_wait:
    halt
    call gray_get_event
    jr z,message_wait

    ret

; **************************************************************************

comp_player:
;    jp user_player

;    ret

exec_player:

    ; put board into correct perspective
    ld hl,(perspective)
    ld a,h                              ; a=player
    cp l                                ; if player!=perspective
    call nz,rotate_board                ; -> rotate board
    call flush_buffer

    call check_player
    jr z,player_lost

    ld (jumpreq),a
    dec a
    jr z,dont_select_piece
    ld a,(select)
    or a
    jr nz,dont_select_piece
    ld a,b
    ld (select),a

dont_select_piece:
    jp user_player
player_2_patch = $-2


#include "gray3.inc"
#include "images2.inc"

#include "d86i.inc"

sprites:
#include "sprites.lzi"

savegame:       .db 0
board:          .db 0,0,0,0,0,0,0,0,0
                .db 0,0,0,0,0,0,0,0,0
                .db 0,0,0,0,0,0,0,0,0
                .db 0,0,0,0,0,0,0,0,0
                .db 0,0,0,0,0,0,0,0,0
                .db 0,0,0,0,0,0,0,0,0
                .db 0,0,0,0,0,0,0,0,0
                .db 0,0,0,0,0,0,0,0,0
perspective:    .db 0
player:         .db 0
p1code:         .dw 0
p2code:         .dw 0
cursor:         .db 0
select:         .db 0
jumpreq:        .db 0
savelen         equ $-savegame

.end


