;gray3.inc
;copyright 1999-2000 joshua seagoe

;gray2 with fifo key event buffer

;on key will kill grayscale and exit to homescreen
;(in case the program crashes)

;                             IMPORTANT:

;make sure you include gray3equ.inc at the start of your program

;memory areas used outside of program:
;$f000-$f3ff light image
;$f400-$f500 interrupt table =$f5
;$f501-$f507 keyboard scan
;$f508   W   incremented on each interrupt
;$f50a-$f50d reserved

;$f50e       3 event buffer head ptr
;$f50f       3 event buffer tail ptr
;$f510-$f52f 3 event buffer

;$f5f5-$f647 interrupt routine
;$fc00-$ffff dark image

;      bit 7     6     5     4     3     2     1     0     event code
;$f507  [ MORE  EXIT  2nd   F1    F2    F3    F4    F5   ]  08+bit#
;$f506  [ ALPHA GRAPH LOG   LN    x^2   ,     STO   --   ]  10
;$f505  [ x-var TABLE SIN   EE    7     4     1     0    ]  18
;$f504  [ DEL   PRGM  COS   (     8     5     2     .    ]  20
;$f503  [ --   CUSTOM TAN   )     9     6     3     (-)  ]  28
;$f502  [ --   CLEAR  ^     /     *     -     +    ENTER ]  30
;$f501  [ --    --    --    --    UP    RIGHT LEFT DOWN  ]  38

gray_install:
        ld hl,gray_int          ;load interrupt routine
        ld de,gray_int_routine
        ld bc,gray_bottom-gray_top
        ldir
        ld hl,gray_int_table    ;load ptr table
        ld de,gray_int_table+1
        ld bc,256
        ld (hl),gray_int_value
        ldir
        ld hl,gray_data_start   ;clear data area
        ld de,gray_data_start+1
        ld bc,gray_data_size
        ld (hl),0
        ldir
        di
        exx
        ld c,0
        ld de,gray_planes
        exx
        ld a,gray_int_table/256
        ld i,a
        im 2
        ei
        ret

gray_remove:
        di
        im 1
        ld a,$3c
        out (0),a
        ld a,($c008)    ; contrast
        out (2),a
        ei
        ret

;begin layer 3 section
#ifndef GRAY_NOLAYER3

;returns with nz if success
gray_add_event:
        ld c,a
gray_add_event_c:
        ld a,(gray_event_tail)
        ld b,a
        ld a,(gray_event_head)  ;insert at head
        dec a
        and $1f                 ;keep in the buffer
        cp b
        ret z                   ;fail if the buffer is full
        ld (gray_event_head),a
        push hl
        ld h,gray_event_buffer/256
        add a,gray_event_buffer&255 ;should be nz
        ld l,a
        ld (hl),c               ;store event
        pop hl
        ret

;returns nz if event is available
;event is in a
gray_get_event:
        push bc
        ld bc,(gray_event_head)
        ld a,b
        cp c
        pop bc
        ret z                   ;fail if buffer is empty
        dec a
        and $1f
        ld (gray_event_tail),a
        push hl
        ld h,gray_event_buffer/256
        add a,gray_event_buffer&255 ;should be nz
        ld l,a
        ld a,(hl)
        pop hl
        ret

#endif
;end layer 3 section

gray_int:

gray_top:
        ex af,af'
        exx
        in a,(3)
        bit 0,a         ;has on been pressed?
        jr nz,gray_shutdown ;yeah, kill the program
        bit 1,a         ;200/s interrupt
        jr z,gray_end
gray_cancel_shutdown:
        ld hl,(gray_time)
        inc hl
        ld (gray_time),hl
        ld a,c
        inc a           ;c=frame counter
        cp gray_total_count
gray_total_patch = $-1-gray_int+gray_int_routine
        jr nz,gray_noreset
        xor a
gray_noreset:
        ld c,a
        cp gray_split_count
gray_split_patch = $-1-gray_int+gray_int_routine
        jr c,gray_doliteframe
        ld a,($c008)    ; contrast
        dec a
        out (2),a
        ld a,d
        jr gray_doframe
gray_doliteframe:
        ld a,($c008)    ; contrast
        inc a           ; darker for light plane
        out (2),a
        ld a,e
gray_doframe:
        out (0),a       ;set visible frame
        ld a,$fe
        push de
        ld hl,gray_keys
        ld b,7
gray_checkkeys:
        ld d,b
        ld e,a
;begin layer 3 section
#ifndef GRAY_NOLAYER3
        ld b,(hl)       ;compare to old key state
        push bc
#endif
;end layer 3 section
        call gray_checkrow
        xor $ff
        ld (hl),a
;begin layer 3 section
#ifndef GRAY_NOLAYER3
        pop bc
        xor b           ;right here
        call nz,gray_key_event
#endif
;end layer 3 section
        ld a,e
        ld b,d
        inc hl
        rlca
        djnz gray_checkkeys
        pop de
        in a,(3)
        and $01
        add a,9
        out (3),a
gray_end:
        ld a,$b
        out (3),a
        exx
        ex af,af'
        ei
        reti

gray_shutdown:
        bit 3,a
        jr z,gray_cancel_shutdown
        im 1
        ld a,$3c
        out (0),a
        ld a,$a
        out (3),a
        inc a
        out (3),a
        ex af,af'
        exx
        ei
        jp $409c        ;jforce
gray_bottom:

;this guy stays here.  identical to rom routine...
gray_checkrow:
        out (1),a       ;write key mask
        nop             ;wait for hardware
        nop
        nop
        nop
        in a,(1)
        ld b,a
        ld a,$ff
        out (1),a       ;tell hardware to go back to sleep
        ld a,b
        ret

;begin layer 3 section
#ifndef GRAY_NOLAYER3

;this is called when the status of a key changes
;d = row number
;a = keys that have changed
;b = old key state  (could use l, it's not backwards)
;--- a&b can change
gray_key_event:
        push bc

        ld c,d
        rl c            ;make room for bit #
        rl c
        rl c
gray_key_event_d:
        rra             ;v- bit is off, skip
        jr nc,gray_key_event_a
        rr b            ;up/down
        jr c,gray_key_event_b
        set 6,c         ;set down marker
gray_key_event_b:
        push bc
        push af
        call gray_add_event_c
        pop af
        pop bc
        res 6,c
        jr gray_key_event_c
gray_key_event_a:
        rr b            ;shift even if key ignored
gray_key_event_c:
        inc c           ;next key in row
        or a            ;any more keys?
        jr nz,gray_key_event_d
        pop bc
        ret

#endif
;end layer 3 section
